@@ -5,24 +5,32 @@
 // <LICENSE-MIT or https://opensource.org/licenses/MIT>, at your
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
-use crate::conversion::analysis::fun::{ArgumentAnalysis, ReceiverMutability};
+use crate::{
+    conversion::analysis::fun::{ArgumentAnalysis, ReceiverMutability},
+    types::QualifiedName,
+};
 use proc_macro2::TokenStream;
 use quote::quote;
-use std::borrow::Cow;
+use std::{borrow::Cow, collections::HashSet};
 use syn::{
     parse_quote, punctuated::Punctuated, token::Comma, FnArg, GenericArgument, PatType, Path,
-    PathSegment, ReturnType, Type, TypePath,
+    PathSegment, ReturnType, Type, TypePath, TypeReference,
 };
 
 /// Function which can add explicit lifetime parameters to function signatures
 /// where necessary, based on analysis of parameters and return types.
-/// This is necessary only in one case - where the parameter is a Pin<&mut T>
-/// and the return type is some kind of reference - because lifetime elision
-/// is not smart enough to see inside a Pin.
+/// This is necessary only in two cases:
+/// 1) where the parameter is a Pin<&mut T>
+///    and the return type is some kind of reference - because lifetime elision
+///    is not smart enough to see inside a Pin.
+/// 2) as a workaround for https://github.com/dtolnay/cxx/issues/1024, where the
+///    input parameter is a non-POD type but the output reference is a POD or
+///    built-in type
 pub(crate) fn add_explicit_lifetime_if_necessary<'r>(
     param_details: &[ArgumentAnalysis],
     mut params: Punctuated<FnArg, Comma>,
     ret_type: &'r ReturnType,
+    non_pod_types: &HashSet<QualifiedName>,
 ) -> (
     Option<TokenStream>,
     Punctuated<FnArg, Comma>,
@@ -31,7 +39,11 @@ pub(crate) fn add_explicit_lifetime_if_necessary<'r>(
     let has_mutable_receiver = param_details
         .iter()
         .any(|pd| matches!(pd.self_type, Some((_, ReceiverMutability::Mutable))));
-    if !has_mutable_receiver {
+
+    let non_pod_ref_param = reference_parameter_is_non_pod_reference(&params, non_pod_types);
+    let ret_type_pod = return_type_is_pod_or_known_type_reference(ret_type, non_pod_types);
+    let hits_1024_bug = non_pod_ref_param && ret_type_pod;
+    if !(has_mutable_receiver || hits_1024_bug) {
         return (None, params, Cow::Borrowed(ret_type));
     }
     let new_return_type = match ret_type {
@@ -65,7 +77,8 @@ pub(crate) fn add_explicit_lifetime_if_necessary<'r>(
                             path: Path { segments, .. },
                             ..
                         }) => add_lifetime_to_pinned_reference(segments).unwrap(),
-                        _ => panic!("Expected Pin<T>"),
+                        Type::Reference(tyr) => add_lifetime_to_reference(tyr),
+                        _ => panic!("Expected Pin<&mut T> or &T"),
                     },
                     _ => panic!("Unexpected fnarg"),
                 }
@@ -76,6 +89,44 @@ pub(crate) fn add_explicit_lifetime_if_necessary<'r>(
     }
 }
 
+fn reference_parameter_is_non_pod_reference(
+    params: &Punctuated<FnArg, Comma>,
+    non_pod_types: &HashSet<QualifiedName>,
+) -> bool {
+    params.iter().any(|param| match param {
+        FnArg::Typed(PatType { ty, .. }) => match ty.as_ref() {
+            Type::Reference(TypeReference { elem, .. }) => match elem.as_ref() {
+                Type::Path(typ) => {
+                    let qn = QualifiedName::from_type_path(typ);
+                    non_pod_types.contains(&qn)
+                }
+                _ => false,
+            },
+            _ => false,
+        },
+        _ => false,
+    })
+}
+
+fn return_type_is_pod_or_known_type_reference(
+    ret_type: &ReturnType,
+    non_pod_types: &HashSet<QualifiedName>,
+) -> bool {
+    match ret_type {
+        ReturnType::Type(_, boxed_type) => match boxed_type.as_ref() {
+            Type::Reference(rtr) => match rtr.elem.as_ref() {
+                Type::Path(typ) => {
+                    let qn = QualifiedName::from_type_path(typ);
+                    !non_pod_types.contains(&qn)
+                }
+                _ => false,
+            },
+            _ => false,
+        },
+        _ => false,
+    }
+}
+
 #[derive(Debug)]
 enum AddLifetimeError {
     WasNotPin,
