@@ -12,7 +12,10 @@ use std::fmt::Display;
 use itertools::Itertools;
 use syn::Ident;
 
-use crate::types::{Namespace, QualifiedName};
+use crate::{
+    known_types,
+    types::{make_ident, Namespace, QualifiedName},
+};
 
 #[derive(Debug, Clone)]
 pub enum ConvertError {
@@ -57,6 +60,7 @@ pub enum ConvertError {
     MethodOfNonAllowlistedType,
     MethodOfGenericType,
     DuplicateItemsFoundInParsing,
+    ConstructorWithOnlyOneParam,
 }
 
 fn format_maybe_identifier(id: &Option<Ident>) -> String {
@@ -110,41 +114,83 @@ impl Display for ConvertError {
             ConvertError::MethodOfNonAllowlistedType => write!(f, "This type was not on the allowlist, so we are not generating methods for it.")?,
             ConvertError::MethodOfGenericType => write!(f, "This type is templated, so we can't generate bindings. We will instead generate bindings for each instantiation.")?,
             ConvertError::DuplicateItemsFoundInParsing => write!(f, "bindgen generated multiple different APIs (functions/types) with this name. autocxx doesn't know how to diambiguate them, so we won't generate bindings for any of them.")?,
+            ConvertError::ConstructorWithOnlyOneParam => write!(f, "bindgen generated a move or copy constructor with an unexpected number of parameters.")?,
         }
         Ok(())
     }
 }
 
+/// Ensures that error contexts are always created using the constructors in this
+/// mod, therefore undergoing identifier sanitation.
 #[derive(Clone)]
-pub(crate) enum ErrorContext {
+struct PhantomSanitized;
+
+/// The context of an error, e.g. whether it applies to a function or a method.
+/// This is used to generate suitable rustdoc in the output codegen so that
+/// the errors can be revealed in rust-analyzer-based IDEs, etc.
+#[derive(Clone)]
+pub(crate) struct ErrorContext(ErrorContextType, PhantomSanitized);
+
+/// All idents in this structure are guaranteed to be something we can safely codegen for.
+#[derive(Clone)]
+pub(crate) enum ErrorContextType {
     Item(Ident),
-    Method {
-        self_ty: Ident,
-        method: Ident,
-    },
-    /// This error means the context where the code would be generated doesn't exist, so there's
-    /// nowhere to attach the error, but we still want to record that we had an error generating
-    /// it.
-    NoCode,
+    SanitizedItem(Ident),
+    Method { self_ty: Ident, method: Ident },
 }
 
 impl ErrorContext {
-    /// Return the ID in the output mod with which this should be associated
-    pub(crate) fn get_id(&self) -> Option<&Ident> {
-        match self {
-            ErrorContext::Item(id) => Some(id),
-            ErrorContext::Method { self_ty, method: _ } => Some(self_ty),
-            ErrorContext::NoCode => None,
+    pub(crate) fn new_for_item(id: Ident) -> Self {
+        match Self::sanitize_error_ident(&id) {
+            None => Self(ErrorContextType::Item(id), PhantomSanitized),
+            Some(sanitized) => Self(ErrorContextType::SanitizedItem(sanitized), PhantomSanitized),
         }
     }
+
+    pub(crate) fn new_for_method(self_ty: Ident, method: Ident) -> Self {
+        // If this IgnoredItem relates to a method on a self_ty which we can't represent,
+        // e.g. u8, then forget about trying to attach this error text to something within
+        // an impl block.
+        match Self::sanitize_error_ident(&self_ty) {
+            None => Self(
+                ErrorContextType::Method {
+                    self_ty,
+                    method: Self::sanitize_error_ident(&method).unwrap_or(method),
+                },
+                PhantomSanitized,
+            ),
+            Some(_) => Self(
+                ErrorContextType::SanitizedItem(make_ident(format!("{}_{}", self_ty, method))),
+                PhantomSanitized,
+            ),
+        }
+    }
+
+    /// Because errors may be generated for invalid types or identifiers,
+    /// we may need to scrub the name
+    fn sanitize_error_ident(id: &Ident) -> Option<Ident> {
+        let qn = QualifiedName::new(&Namespace::new(), id.clone());
+        if known_types().conflicts_with_built_in_type(&qn) {
+            Some(make_ident(format!("{}_autocxx_error", qn.get_final_item())))
+        } else {
+            None
+        }
+    }
+
+    pub(crate) fn get_type(&self) -> &ErrorContextType {
+        &self.0
+    }
+
+    pub(crate) fn into_type(self) -> ErrorContextType {
+        self.0
+    }
 }
 
 impl std::fmt::Display for ErrorContext {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self {
-            ErrorContext::Item(id) => write!(f, "{}", id),
-            ErrorContext::Method { self_ty, method } => write!(f, "{}::{}", self_ty, method),
-            ErrorContext::NoCode => write!(f, "<not generated>"),
+        match &self.0 {
+            ErrorContextType::Item(id) | ErrorContextType::SanitizedItem(id) => write!(f, "{}", id),
+            ErrorContextType::Method { self_ty, method } => write!(f, "{}::{}", self_ty, method),
         }
     }
 }
