@@ -64,13 +64,6 @@ pub use parse_file::{parse_file, ParseError, ParsedFile};
 
 pub use cxx_gen::HEADER;
 
-/// Re-export cxx such that clients can use the same version as
-/// us. This doesn't enable clients to avoid depending on the cxx
-/// crate too, unfortunately, since generated cxx::bridge code
-/// refers explicitly to ::cxx. See
-/// <https://github.com/google/autocxx/issues/36>
-pub use cxx;
-
 #[derive(Clone)]
 /// Some C++ content which should be written to disk and built.
 pub struct CppFilePair {
